@@ -27,7 +27,7 @@ the mpd module:
 
 LOWEST_NOTENAME="c,,,,"
 HIGHEST_NOTENAME="g''''''"
-from const import FIRST, LAST, ALL
+from const import FIRST, LAST, ALL, DEFAULT_VOLUME, DEFAULT_VELOCITY
 
 import parser
 import soundcard
@@ -35,36 +35,59 @@ from musicalpitch import MusicalPitch, InvalidNotenameException
 from interval import Interval
 from mpdutils import notename_to_int, int_to_notename, int_to_user_notename
 from rat import Rat
-from track import Track
+from track import Track, PercussionTrack
 from _exceptions import MpdException
 
-def music_to_tracklist(music, velocity, start=None, end=None):
+def music_to_tracklist(music, start=None, end=None):
     """
     return a list of tracks, where track[0] use only channel 0,
     track[1] only use channel 1 etc.
     """
     score = parser.parse_to_score_object(music)
-    tracks = score.get_midi_events(velocity, start, end)
+    tracks = score.get_midi_events(start, end)
     return tracks
 
-def music_to_track(music, velocity, start=None, end=None):
+def music_to_track(music, start=None, end=None):
     score = parser.parse_to_score_object(music)
-    tracklist = score.get_midi_events(velocity, start, end)
+    tracklist = score.get_midi_events(start, end)
     track = tracklist[0]
     for x in range(1, len(tracklist)):
         track.merge_with(tracklist[x])
     return track
 
-def play_music(music, tempo, patch, velocity, start=None, end=None):
+def play_note(patch, notelen, pitch, volume=DEFAULT_VOLUME,
+              velocity=DEFAULT_VELOCITY):
+    assert isinstance(patch, int) and 0 <= patch < 128
+    assert isinstance(notelen, int)
+    assert isinstance(pitch, int) and 0 <= pitch < 128
+    assert isinstance(velocity, int) and 0 <= velocity < 128
+    t = Track()
+    t.set_volume(volume)
+    t.set_patch(patch)
+    t.note(notelen, pitch, velocity)
+    soundcard.synth.play_track(t)
+
+def play_perc(notelen, pitch, volume=DEFAULT_VOLUME,
+              velocity=DEFAULT_VELOCITY):
+    assert isinstance(notelen, int)
+    assert isinstance(pitch, int) and 0 <= pitch < 128
+    assert isinstance(velocity, int) and 0 <= velocity < 128
+    t = PercussionTrack()
+    t.set_volume(volume)
+    t.note(notelen, pitch, velocity)
+    soundcard.synth.play_track(t)
+
+def play_music(music, tempo, patch, volume, start=None, end=None):
     if type(tempo) == type(0):
         bpm = tempo
         nl = 4
     else:
         bpm, nl = tempo
     score = parser.parse_to_score_object(music)
-    tracklist = score.get_midi_events(velocity, None, None)
+    tracklist = score.get_midi_events(None, None)
     tracklist[0].prepend_bpm(bpm, nl)
     [track.prepend_patch(patch) for track in tracklist]
+    [track.prepend_volume(volume) for track in tracklist]
     soundcard.synth.play_track(*tracklist)
 
 def play_music3(music, tempo, instrument, start=None, end=None):
@@ -74,16 +97,17 @@ def play_music3(music, tempo, instrument, start=None, end=None):
     """
     if len(instrument) == 2:
         instrument = instrument * 3
-    # FIXME, only the velocity from instrument[1] is used since
-    # music_to_tracklist is not complete.
-    tracklist = music_to_tracklist(music, instrument[1], start, end)
+    tracklist = music_to_tracklist(music, start, end)
     tracklist[0].prepend_patch(instrument[4])
+    tracklist[0].prepend_volume(instrument[5])
     a, b = tempo
     # FIXME the bpm api is wrong. We set the tempo to one track, and it works for all.
     tracklist[0].prepend_bpm(a, b)
     for track in tracklist[:-1][1:]:
         track.prepend_patch(instrument[2])
+        track.prepend_volume(instrument[3])
     tracklist[-1].prepend_patch(instrument[0])
+    tracklist[-1].prepend_volume(instrument[1])
     soundcard.synth.play_track(*tracklist)
 
 ##################
