@@ -55,8 +55,11 @@ def write_int32(f, i):
     
 MIDI_NOTE_OFF = 0x80
 MIDI_NOTE_ON = 0x90
+MIDI_CONTROLLER_CHANGE = 0xb0
 MIDI_PROGRAM_CHANGE = 0xc0
 
+MIDI_CONTROLLER_VOLUME = 0x07
+
 def mf_tempo(n):
     """
     n -- number of quarter tones per minute.
@@ -75,6 +78,12 @@ def mf_timesig(numerator, denuminator):
 def mf_program_change(chan, prg):
     assert 0 <= chan < 16
     return mf_delta(0) + [chan+MIDI_PROGRAM_CHANGE, prg]
+
+def mf_volume_change(chan, volume):
+    assert 0 <= chan < 16
+    assert 0 <= volume < 256
+    return mf_delta(0) + [chan + MIDI_CONTROLLER_CHANGE, MIDI_CONTROLLER_VOLUME, volume]
+
 def mf_note_on(delta, chan, note, vel):
     assert 0 <= chan < 16
     return mf_delta(delta) + [chan+MIDI_NOTE_ON, note, vel]
