@@ -16,6 +16,7 @@
 
 from rat import Rat
 import mfutils
+from const import DEFAULT_VELOCITY, DEFAULT_VOLUME
 
 class EventBase(object):
     def __init__(self):
@@ -58,6 +59,13 @@ class SetPatchEvent(EventBase):
     def __str__(self):
         return "(%s, time:%s, patch:%i)" % ( self.__class__.__name__, self.m_time, self.m_patch)
 
+class SetVolumeEvent(EventBase):
+    def __init__(self, volume):
+        EventBase.__init__(self)
+        assert 0 <= volume < 256
+        self.m_volume = int(volume)
+    def __str__(self):
+        return "(%s, time:%s, volume:%i)" % ( self.__class__.__name__, self.m_time, self.m_volume)
 
 class TempoEvent(EventBase):
     def __init__(self, bpm, notelen):
@@ -67,6 +75,7 @@ class TempoEvent(EventBase):
 
 class MidiEventStream(object):
     TEMPO = 'tempo'
+    VOLUME = 'volume'
     NOTE_ON = 'note-on'
     NOTE_OFF = 'note-off'
     NOTELEN_TIME = 'notelen-time'
@@ -99,7 +108,7 @@ class MidiEventStream(object):
             for i in range(self.percussion_MIDI_channel) \
                      + range(self.percussion_MIDI_channel, 16):
                 self.free_MIDI_channels.append(self.MidiChannel(i))
-            # The dict key will be the patch number.
+            # The dict key will be the (patch number, volume).
             # The value is a MidiChannel object.
             self.allocated_MIDI_channels = {}
             # This dict maps from MIDI channel number to the actual
@@ -107,28 +116,34 @@ class MidiEventStream(object):
             self.int_to_channel_object = {}
             for channel in self.free_MIDI_channels:
                 self.int_to_channel_object[channel.m_number] = channel
-        def alloc_channel(self, patch):
+        def set_channel_data(self, channel, key, data):
+            setattr(self.int_to_channel_object[channel], key, data)
+        def get_channel_data(self, channel, key):
+            return getattr(self.int_to_channel_object[channel], key, None)
+        def alloc_channel(self, patch, volume):
             """
             Allocate a midi channel for the patch and
             return the MIDI channel number of the allocated channel.
             """
             #FIXME need to handle running out of available midi channels.
-            assert patch not in self.allocated_MIDI_channels
-            for num, channel in self.allocated_MIDI_channels.items():
+            key = (patch, volume)
+            assert key not in self.allocated_MIDI_channels
+            # First try to find an allocated channel that is silent.
+            for k, channel in self.allocated_MIDI_channels.items():
                 if channel.is_silent():
-                    self.allocated_MIDI_channels[patch] = \
-                        self.allocated_MIDI_channels[num]
-                    del self.allocated_MIDI_channels[num]
-                    return self.allocated_MIDI_channels[patch].m_number
-            self.allocated_MIDI_channels[patch] = self.free_MIDI_channels.pop(0)
-            return self.allocated_MIDI_channels[patch].m_number
-        def get_channel_for_patch(self, patch):
+                    self.allocated_MIDI_channels[key] = \
+                        self.allocated_MIDI_channels[k]
+                    del self.allocated_MIDI_channels[k]
+                    return self.allocated_MIDI_channels[key].m_number
+            self.allocated_MIDI_channels[key] = self.free_MIDI_channels.pop(0)
+            return self.allocated_MIDI_channels[key].m_number
+        def get_channel_for_patch(self, patch, volume):
             """
             Return the MIDI channel number we should use to play a tone
             with this patch number. Raise KeyError if no channel is allocated
             yet.
             """
-            return self.allocated_MIDI_channels[patch].m_number
+            return self.allocated_MIDI_channels[patch, volume].m_number
         def start_note(self, channel, pitch):
             self.int_to_channel_object[channel].start_tone(pitch)
         def stop_note(self, channel, pitch):
@@ -139,14 +154,13 @@ class MidiEventStream(object):
         self.m_tracks = tracks
         for track in self.m_tracks:
             track.calculate_event_times()
-    def __iter__(self):
-        for e in self.generate_track_events():
-            yield e
     def _create_dict_of_track(self, track):
-        # create a dict of the track, where the key is a list with
-        # all events with the same m_time variable.
-        # The dict values are a dict with three keys:
-        # NoteOffEvents, OtherEvents, NoteOnEvents
+        """
+        Return a dict of the track, where the key is a list with
+        all events with the same m_time variable.
+        The dict values are a dict with three keys:
+        NoteOffEvents, OtherEvents, NoteOnEvents
+        """
         retval = {}
         for event in track.m_v:
             retval[event.m_time] = retval.get(event.m_time, [])
@@ -157,8 +171,7 @@ class MidiEventStream(object):
               'OtherEvents': [x for x in retval[key] if not isinstance(x, (NoteOffEvent, NoteOnEvent))],
               'NoteOnEvents': [x for x in retval[key] if isinstance(x, NoteOnEvent)]}
         return retval
-    def generate_track_events(self):
-        #FIXME find better method name
+    def __iter__(self):
         # tpos_set will know all the positions in time where anything happens
         # on any staff
         tpos_set = set()
@@ -169,7 +182,12 @@ class MidiEventStream(object):
         tpos_list.sort()
         # We use this variable to remember which instrument
         # we want the track to play.
-        track_instrument = [0] * len(self.m_tracks)
+        track_state = {}
+        for x in range(len(self.m_tracks)):
+            track_state[x] = {'volume-requested': DEFAULT_VOLUME,
+                        'patch-requested': 0}
+        tempo_request = None
+        tempo_current = None
         # We use this list of dicts to know which tones are playing
         # which patch on the tracks. The key of the dicts are the integer
         # value representing the pitch, and the value is the patch number.
@@ -194,7 +212,7 @@ class MidiEventStream(object):
                         if isinstance(self.m_tracks[idx], PercussionTrack):
                             chn = ch_dev.percussion_MIDI_channel
                         else:
-                            chn = ch_dev.get_channel_for_patch(patch)
+                            chn = ch_dev.get_channel_for_patch(patch, track_state[idx]['volume-requested'])
                         if ch_dev.is_playing(chn, e.m_pitch):
                             ch_dev.stop_note(chn, e.m_pitch)
                             yield self.NOTE_OFF, chn, e.m_pitch, e.m_velocity
@@ -202,29 +220,44 @@ class MidiEventStream(object):
                 if tpos in track:
                     for e in track[tpos]['OtherEvents']:
                         if isinstance(e, SetPatchEvent):
-                            track_instrument[idx] = e.m_patch
+                            track_state[idx]['patch-requested'] = e.m_patch
+                        elif isinstance(e, SetVolumeEvent):
+                            track_state[idx]['volume-requested'] = e.m_volume
                         elif isinstance(e, TempoEvent):
-                            yield self.TEMPO, e.m_bpm, e.m_notelen
+                            tempo_request = e
                         else:
                             print "NOT HANDLING EVENT:", e
             for idx, track in enumerate(tracks2):
                 if tpos in track:
                     for e in track[tpos]['NoteOnEvents']:
                         assert e.m_pitch not in track_notes[idx]
+                        if tempo_request != tempo_current:
+                            yield self.TEMPO, tempo_request.m_bpm, tempo_request.m_notelen
+                            tempo_current = tempo_request
                         if isinstance(self.m_tracks[idx], PercussionTrack):
                             chn = ch_dev.percussion_MIDI_channel
+                            if ch_dev.get_channel_data(chn, 'volume') != track_state[idx]['volume-requested']:
+                                ch_dev.set_channel_data(chn, 'volume', track_state[idx]['volume-requested'])
+                                yield self.VOLUME, chn, track_state[idx]['volume-requested']
                         else:
                             try:
-                                chn = ch_dev.get_channel_for_patch(track_instrument[idx])
+                                chn = ch_dev.get_channel_for_patch(
+                                        track_state[idx]['patch-requested'], 
+                                        track_state[idx]['volume-requested'])
                             except KeyError:
-                                chn = ch_dev.alloc_channel(track_instrument[idx])
-                                # We will only yield SET_PATCH if a channel
-                                # was allocated.
-                                yield self.SET_PATCH, chn, track_instrument[idx]
+                                chn = ch_dev.alloc_channel(
+                                        track_state[idx]['patch-requested'],
+                                        track_state[idx]['volume-requested'])
+                            if ch_dev.get_channel_data(chn, 'patch') != track_state[idx]['patch-requested']:
+                                ch_dev.set_channel_data(chn, 'patch', track_state[idx]['patch-requested'])
+                                yield self.SET_PATCH, chn, track_state[idx]['patch-requested']
+                            if ch_dev.get_channel_data(chn, 'volume') != track_state[idx]['volume-requested']:
+                                ch_dev.set_channel_data(chn, 'volume', track_state[idx]['volume-requested'])
+                                yield self.VOLUME, chn, track_state[idx]['volume-requested']
                         if ch_dev.is_playing(chn, e.m_pitch):
                             print "info: ignoring duplicate tone:", e
                             continue
-                        track_notes[idx][e.m_pitch] = track_instrument[idx]
+                        track_notes[idx][e.m_pitch] = track_state[idx]['patch-requested']
                         # ch_dev must know which tones are sounding on which
                         # MIDI channels, so it can handle the midi resources.
                         ch_dev.start_note(chn, e.m_pitch)
@@ -254,6 +287,8 @@ class MidiEventStream(object):
                     notelen = 0
                 elif e[0] == self.SET_PATCH:
                     v = v + mfutils.mf_program_change(e[1], e[2])
+                elif e[0] == self.VOLUME:
+                    v = v + mfutils.mf_volume_change(e[1], e[2])
                 elif e[0] == self.BENDER:
                     print "FIXME todo: seq_bender for play_with_drvmidi"
                     #m.seq_bender(DEV, e[1], e[2])
@@ -268,36 +303,26 @@ class MidiEventStream(object):
         f.close()
     def str_repr(self, details=0):
         v = []
-        v.append([])
         for e in self:
             if e[0] == self.TEMPO:
-                v[-1].append("t%s/%s" % (e[1], e[2]))
+                v.append("t%s/%s" % (e[1], e[2]))
             elif e[0] == self.NOTE_ON:
                 if e[1] == 9:
-                    v[-1].append("P%s" % e[2])
+                    v.append("P%s" % e[2])
                 else:
                     if details == 0:
-                        v[-1].append("n%s" % e[2])
+                        v.append("n%s" % e[2])
                     elif details == 1:
-                        v[-1].append("n%s:%s" % (e[1], e[2]))
+                        v.append("n%s:%s" % (e[1], e[2]))
             elif e[0] == self.NOTE_OFF:
-                v[-1].append("o%s" % e[2])
+                v.append("o%s" % e[2])
             elif e[0] == self.SET_PATCH:
-                v[-1].append("p%i:%i" % (e[1], e[2]))
+                v.append("p%i:%i" % (e[1], e[2]))
+            elif e[0] == self.VOLUME:
+                v.append("v%i:%i" % (e[1], e[2]))
             elif e[0] == self.NOTELEN_TIME:
-                v[-1].append("d%i/%i" % (e[1].m_num, e[1].m_den))
-                v.append([])
-        retval = []
-        for x in v:
-            n = [e for e in x if e.startswith('t')]
-            if n:
-                retval.append(" ".join([e for e in x if e.startswith('t')]))
-            for typ in ('p', 'o', 'P', 'n', 'd'):
-                n = [e for e in x if e.startswith(typ)]
-                if n:
-                    n.sort()
-                    retval.append(" ".join(n))
-        return " ".join(retval)
+                v.append("d%i/%i" % (e[1].m_num, e[1].m_den))
+        return " ".join(v)
 
 
 class Track:
@@ -316,6 +341,10 @@ class Track:
         for e in self.m_v:
             if isinstance(e, SetPatchEvent):
                 retval.append('p%i' % e.m_patch)
+            elif isinstance(e, TempoEvent):
+                retval.append('t%i/%i' % (e.m_bpm, e.m_notelen))
+            elif isinstance(e, SetVolumeEvent):
+                retval.append('v%i' % e.m_volume)
             elif isinstance(e, NoteOnEvent):
                 retval.append('n%i' % e.m_pitch)
             elif isinstance(e, NoteOffEvent):
@@ -323,14 +352,22 @@ class Track:
             elif isinstance(e, Delay):
                 retval.append('d%i/%i' % (e.m_duration.m_num, e.m_duration.m_den))
         return " ".join(retval)
-    def __init__(self):
+    def __init__(self, default_velocity=None):
+        if default_velocity is None:
+            self.m_default_velocity = DEFAULT_VELOCITY
+        else:
+            self.m_default_velocity = default_velocity
         self.m_v = []
-    def start_note(self, pitch, vel):
+    def start_note(self, pitch, vel=None):
         assert 0 <= int(pitch) < 128
+        if vel is None:
+            vel = self.m_default_velocity
         assert 0 <= vel < 128
         self.m_v.append(NoteOnEvent(int(pitch), int(vel)))
-    def stop_note(self, pitch, vel):
+    def stop_note(self, pitch, vel=None):
         assert 0 <= int(pitch) < 128
+        if vel is None:
+            vel = self.m_default_velocity
         assert 0 <= vel < 128
         self.m_v.append(NoteOffEvent(int(pitch), int(vel)))
     def notelen_time(self, notelen):
@@ -346,10 +383,13 @@ class Track:
         else:
             assert isinstance(notelen, Rat)
             self.m_v.append(Delay(notelen))
-    def note(self, notelen, pitch, vel):
+    def note(self, notelen, pitch, vel=None):
         """
         See notelen_time docstring.
         """
+        if vel is None:
+            vel = self.m_default_velocity
+        assert 0 <= vel < 128
         self.start_note(pitch, vel)
         self.notelen_time(notelen)
         self.stop_note(pitch, vel)
@@ -366,6 +406,10 @@ class Track:
         only the first call will have any effect.
         """
         self.m_v.insert(0, SetPatchEvent(patch))
+    def set_volume(self, volume):
+        self.m_v.append(SetVolumeEvent(volume))
+    def prepend_volume(self, volume):
+        self.m_v.insert(0, SetVolumeEvent(volume))
     def set_bpm(self, bpm, notelen=4):
         self.m_v.append(TempoEvent(bpm, notelen))
     def prepend_bpm(self, bpm, notelen=4):
@@ -373,11 +417,6 @@ class Track:
     def bender(self, chn, value):
         "value >= 0"
         self.m_v.append([self.BENDER, chn, value])
-    def create_merge(self, track1, track2):
-        """
-        Return a new track that merges track1 and track2.
-        """
-    create_merge = staticmethod(create_merge)
     def merge_with(self, B):
         D = {}
         for track in [self, B]:
