@@ -69,5 +69,5 @@ class WinSynth(SynthCommon):
                 print "ugh todo: seq_bender for play_with_drvmidi"
                 #m.seq_bender(DEV, e[1], e[2])
             else:
-                raise error
+                pass
         self.__driver.play()
