@@ -447,7 +447,7 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
     def _add_auto_new_question_gui(self, box):
         hbox = gu.bHBox(box, False)
         hbox.set_spacing(gu.PAD_SMALL)
-        adj = gtk.Adjustment(0, 0, 10, 0.1, 1, 1)
+        adj = gtk.Adjustment(0, 0, 10, 0.1, 1)
         spin = gu.nSpinButton(self.m_exname, 'seconds_before_new_question',
                        adj)
         spin.set_digits(1)
