@@ -23,7 +23,6 @@ import abstract
 import statistics
 import statisticsviewer
 import mpd
-import utils
 
 class Teacher(abstract.Teacher):
     OK = 0
@@ -76,7 +75,7 @@ class Gui(abstract.LessonbasedGui):
     def __init__(self, teacher, window):
         abstract.LessonbasedGui.__init__(self, teacher, window)
         self.g_lesson_heading.hide()
-        self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer(utils.play_tone)
+        self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer()
         self.practise_box.pack_start(self.g_music_displayer, False)
 
         self.g_answer_button_box = gu.NewLineBox()
