@@ -18,6 +18,7 @@ import gtk, gobject
 import soundcard
 import abstract, statistics, statisticsviewer, gu, random
 import lessonfile
+import utils
 
 class Teacher(abstract.Teacher):
     def __init__(self, exname, app):
@@ -71,7 +72,7 @@ class Teacher(abstract.Teacher):
             gobject.source_remove(self.m_timeout_tag)
             self.m_timeout_tag = None
     def tick(self, _o=None):
-        soundcard.play_perc(8, 60, 90)
+        utils.play_perc(8, 60)
         return 1
     def guess_answer(self, bpm):
         assert self.q_status != self.QSTATUS_NO
