@@ -106,11 +106,17 @@ def new_labelobject(label):
         QuestionNameButtonTable and make it transpose the labels.
         But I don't think translated labels should have high priority.
     """
-    if isinstance(label, unicode):
+    if isinstance(label, basestring):
+        # We hope all strings are unicode, but check for basestring just
+        # in case some modules are wrong.
         l = gtk.Label(label)
         l.show()
     else:
         if isinstance(label, tuple):
+            # FIXME this _probably_never happens. I think only old code
+            # that has  been removed. But let us keep the code until
+            # we have a chanse to review.
+            print "WARNING: lessonfilegui.new_labelobject: label is a tuple."
             labeltype, labeldata = label
         else:
             labeltype = label.m_labeltype
