@@ -4,6 +4,8 @@ import sys
 import subprocess
 import signal
 import re
+import filesystem
+
 if sys.platform == 'win32':
     import ctypes
 
@@ -166,34 +168,58 @@ def find_progs(execs):
 
     execs - a tuple of possible executable names
     """
-    retval = []
+    assert isinstance(execs, (list, tuple,))
+    retval = set()
     for p in os.environ['PATH'].split(os.pathsep):
         for e in execs:
             ex = os.path.join(p, e)
-            if os.path.exists(ex) and ex not in retval:
-                retval.append(ex)
+            if os.path.isfile(ex):
+                retval.add(ex)
     return retval
 
 
 def find_mma_executables(ignore_drives):
     """
-    Return a list of command lines that we think will run MMA.
+    Return a set of command lines that we think will run MMA.
+    ignore_drives is only used on win32. It is ignored on other operating
+    systems.
     """
     if sys.platform != 'win32':
         return find_progs(("mma",))
     else:
-        retval = []
+        retval = set()
         for drive in get_drives():
             if drive.upper() in ignore_drives:
                 continue
-            for dirname in os.listdir(drive):
-                if dirname.lower().startswith('mma'):
-                    dir = os.path.join(drive, dirname)
-                    mma_py = os.path.join(drive, dirname, "mma.py")
-                    if os.path.exists(mma_py):
-                        retval.append("%s %s" % (sys.executable, mma_py))
-                    mma_bat = os.path.join(drive, dirname, "mma.bat")
-                    if os.path.exists(mma_bat):
-                        retval.append(mma_bat)
+            try:
+                for dirname in os.listdir(drive):
+                    if dirname.lower().startswith('mma'):
+                        dir = os.path.join(drive, dirname)
+                        mma_py = os.path.join(drive, dirname, "mma.py")
+                        if os.path.exists(mma_py):
+                            retval.add(mma_py)
+                        mma_bat = os.path.join(drive, dirname, "mma.bat")
+                        if os.path.exists(mma_bat):
+                            retval.add(mma_bat)
+            except WindowsError:
+                # People have had this exception raised:
+                # WindowsError: [Error 21] The device is not ready: 'X:\\*.*'
+                # So we choose to continue with the next drive if anything
+                # unexcepted happens. 
+                continue
+    return retval
+
+def find_csound_executables():
+    """
+    Return a list of possible csound executables.
+    Currently we return the name of the csound binary found as
+    %PROGRAMFILES%\csound\bin\csound.exe
+    """
+    retval = find_progs(('csound.exe', 'csound'))
+    if sys.platform == 'win32':
+        fn = os.path.join(filesystem.win32_program_files_folder(),
+            "csound", "bin", "csound.exe")
+        if os.path.exists(fn):
+            retval.add(fn)
     return retval
 
