@@ -26,12 +26,8 @@ class Teacher(abstract.Teacher):
     def new_question(self):
         self.q_status = self.QSTATUS_NEW
         self.m_P.select_random_question()
-    def play_answer(self):
-        self.play_question()
     def play_440hz(self):
-        soundcard.play_note(self.get_int('config/preferred_instrument'),
-                       4, mpd.notename_to_int("a'"),
-                       self.get_int('config/preferred_instrument_velocity'))
+        utils.play_note(4, mpd.notename_to_int("a'"))
 
 class Gui(abstract.LessonbasedGui):
     def __init__(self, teacher, window):
@@ -39,7 +35,7 @@ class Gui(abstract.LessonbasedGui):
         ################
         # practise_box #
         ################
-        self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer(utils.play_tone)
+        self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer()
         self.practise_box.pack_start(self.g_music_displayer)
 
         self.g_new = gu.bButton(self.action_area, _("_New"), self.new_question)
