@@ -23,6 +23,7 @@ import gu
 import buildinfo
 from dataparser import Dataparser
 import lessonfile
+import osutils
 
 
 # The definition of a training set will be stored in a file in the
@@ -353,10 +354,14 @@ class TrainingSetDialog(gu.EditorDialogBase):
         time.sleep(0.1)
         while gtk.events_pending():
             gtk.main_iteration(0)
-        for prog in self.m_app.export_training_set(v, export_to, output_format):
-            progress_bar.set_fraction(prog)
-            while gtk.events_pending():
-                gtk.main_iteration(0)
-        progress_dialog.destroy()
+        try:
+            for prog in self.m_app.export_training_set(v, export_to, output_format):
+                progress_bar.set_fraction(prog)
+                while gtk.events_pending():
+                    gtk.main_iteration(0)
+            progress_dialog.destroy()
+        except osutils.BinaryBaseException, e:
+            progress_dialog.destroy()
+            self.m_app.m_ui.display_error_message2(e.msg1, e.msg2)
 
 
